"""Init for Indego class."""
from pkg_resources import DistributionNotFound, get_distribution

try:
    dist_name = __name__
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:
    __version__ = "unknown"
finally:
    del get_distribution, DistributionNotFound

from .indego_async_client import IndegoAsyncClient
from .indego_client import IndegoClient



import json
import logging
import pytz
import pytz    # $ pip install pytz
import tzlocal # $ pip install tzlocal
from pyIndego import IndegoClient
from datetime import datetime
import time

logging.basicConfig(filename="pyIndego.log", level=logging.DEBUG)
_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)


def main(config):
    with IndegoClient(**config) as indego:
        # print(" ")
        # print("=[indego.download_map]===")
        # indego.download_map()
        # print("map: ", indego.map_filename)

        print(" ")
        print("=[indego.update_state]===")
        indego.update_state()
        print(indego.state)
        print(f"State: {indego.state_description}")
        print(f"State detail: {indego.state_description_detail}")

        print(f"xPos: {indego.state.xPos}")
        print(f"yPos: {indego.state.yPos}")

        print(f"Serial: {indego.serial}")
        #print(" ")
        #print("=[update_state longpoll]===")
        #indego.update_state(longpoll=True, longpoll_timeout=20)
        #print(indego.state)
        #print(indego.state_description)
        #print(indego.state_description_detail)

        #print(" ")
        #print("=[indego.update_alerts]====")
        #indego.update_alerts()
        #print(indego.alerts)
        #print("=[indego.alerts_count]====")
        #print(indego.alerts_count)
        #print("=[indego.alerts[0]]====")
        #print(indego.alerts[0])
        #print("=[indego.alerts[1]]====")
        #print(indego.alerts[1])
        #print("=[indego.alerts[2]]====")
        #print(indego.alerts[2])
        #print("=[indego.alerts[3]]====")
        #print(indego.alerts[3])

        # Wakes mower!
        # print(" ")
        # print("=[indego.update_config]====")
        # indego.update_config()
        # print(indego.config)

        #print(" ")
        #print("=[indego.update_generic_data]====")
        #indego.update_generic_data()
        #print(indego.generic_data)
        #print("Generic data min voltage: ", indego.generic_data.model_voltage.min)
        #print(indego.generic_data.alm_mode)

        print(" ")
        print("=[indego.update_last_mow]====")
        indego.update_last_mow()
        print(indego.last_mow)

        #print(" ")
        #print("=[update_location]====")
        #indego.update_location()
        #print(indego.location)

        # print(" ")
        # print("=[indego.update_network]===")
        # indego.update_network()
        # print(indego.network)

        #print(" ")
        #print("=[indego.update_next_mow]===")
        #indego.update_next_mow()
        #print(indego.next_mow)
        
        # Wakes mower!
        # # print(" ")
        # print("=[operating data]====")
        # indego.update_operating_data()
        # print(indego.operating_data)
#
        #print(" ")
        #print("=[indego.update_security]===")
        #indego.update_security()
        #print(indego.security)

        # Wakes mower!
        # print(" ")
        # print("=[indego.update_setup]===")
        # indego.update_setup()
        # print(indego.setup)

        #print(" ")
        #print("=[updates]====")
        #indego.update_updates_available()
        #print(indego.update_available)
        
        #print(" ")
        #print("=[user]====")
        #indego.update_user()
        #print(indego.user)


if __name__ == "__main__":
    with open("config.json", "r") as config_file:
        config = json.load(config_file)
    
    for i in range(10):
        now = datetime.now()
        current_time = now.strftime("%H:%M:%S")
        print(f"=[{current_time}]=================================================================================================")
        print(f"Iteration:  {i+1}")
        main(config)

        print(f"End iteration:  {i+1}")
        now = datetime.now()
        current_time = now.strftime("%H:%M:%S")
        print(f"=[{current_time}]=================================================================================================")
        print(" ")
        print("Sleep for 60 seconds")


        time.sleep(60)





"""Constants for pyIndego."""
from enum import Enum


class Methods(Enum):
    """Enum with HTTP methods."""

    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"
    PATCH = "PATCH"
    OPTIONS = "OPTIONS"
    HEAD = "HEAD"


DEFAULT_URL = "https://api.indego.iot.bosch-si.com/api/v1/"
CONTENT_TYPE_JSON = "application/json"
CONTENT_TYPE = "Content-Type"
DEFAULT_BODY = {
    "device": "",
    "os_type": "Android",
    "os_version": "4.0",
    "dvc_manuf": "unknown",
    "dvc_type": "unknown",
}
COMMANDS = ("mow", "pause", "returnToDock")

DEFAULT_HEADER = {CONTENT_TYPE: CONTENT_TYPE_JSON}
DEFAULT_LOOKUP_VALUE = "Not in database."

DEFAULT_CALENDAR = {
    "sel_cal": 1,
    "cals": [
        {
            "cal": 1,
            "days": [
                {
                    "day": 0,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 1,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 2,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 3,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 4,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 5,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 6,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 23, "EnMin": 59}
                    ],
                },
            ],
        }
    ],
}

MOWER_STATE_DESCRIPTION_DETAIL = {
    0: "Reading status",
    101: "Mower lifted",
    257: "Charging",
    258: "Docked",
    259: "Docked - Software update",
    260: "Charging",
    261: "Docked",
    262: "Docked - Loading map",
    263: "Docked - Saving map",
    266: "Docked - Leaving dock",
    512: "Mowing - Leaving dock",
    513: "Mowing",
    514: "Mowing - Relocalising",
    515: "Mowing - Loading map",
    516: "Mowing - Learning lawn",
    517: "Mowing - Paused",
    518: "Border cut",
    519: "Idle in lawn",
    520: "Mowing - Learning lawn paused",
    521: "Border cut",
    523: "Mowing - Spot mowing",
    524: "Mowing - Random",
    525: "Mowing - Random complete",
    768: "Returning to Dock",
    769: "Returning to Dock",
    770: "Returning to Dock",
    771: "Returning to Dock - Battery low",
    772: "Returning to dock - Calendar timeslot ended",
    773: "Returning to dock - Battery temp range",
    774: "Returning to dock - requested by user/app",
    775: "Returning to dock - Lawn complete",
    776: "Returning to dock - Relocalising",
    1005: "Connection to dockingstation failed",
    1025: "Diagnostic mode",
    1026: "End of life",
    1027: "Service Requesting Status",
    1038: "Mower immobilized",
    1281: "Software update",
    1537: "Stuck on lawn, help needed",
    64513: "Sleeping",
    99999: "Offline",
}


MOWER_STATE_DESCRIPTION = {
    0: "Docked",
    101: "Docked",
    257: "Docked",
    258: "Docked",
    259: "Docked",
    260: "Docked",
    261: "Docked",
    262: "Docked",
    263: "Docked",
    266: "Mowing",
    512: "Mowing",
    513: "Mowing",
    514: "Mowing",
    515: "Mowing",
    516: "Mowing",
    517: "Mowing",
    518: "Mowing",
    519: "Mowing",
    520: "Mowing",
    521: "Mowing",
    522: "Mowing",
    523: "Mowing",
    524: "Mowing",
    525: "Mowing",
    768: "Mowing",
    769: "Mowing",
    770: "Mowing",
    771: "Mowing",
    772: "Mowing",
    773: "Mowing",
    774: "Mowing",
    775: "Mowing",
    776: "Mowing",
    1005: "Mowing",
    1025: "Diagnostic mode",
    1026: "End of life",
    1027: "Service Requesting Status",
    1038: "Mower immobilized",
    1281: "Software update",
    1537: "Stuck",
    64513: "Docked",
    99999: "Offline",
}

MOWER_MODEL_DESCRIPTION = {
    "3600HA2300": "Indego 1000",
    "3600HA2301": "Indego 1200",
    "3600HA2302": "Indego 1100",
    "3600HA2303": "Indego 13C",
    "3600HA2304": "Indego 10C",
    "3600HB0100": "Indego 350",
    "3600HB0101": "Indego 400",
    "3600HB0102": "Indego S+ 350 1gen",
    "3600HB0103": "Indego S+ 400 1gen",
    "3600HB0105": "Indego S+ 350 2gen",
    "3600HB0106": "Indego S+ 400 2gen",
    "3600HB0302": "Indego S+ 500",
    "3600HB0301": "Indego M+ 700",
}

MOWING_MODE_DESCRIPTION = {
    "smart": "SmartMowing",
    "calendar": "Calendar",
    "manual": "Manual",
}

ALERT_ERROR_CODE = {
    "104": "Stop button pushed",
    "101": "Mower lifted",
    "115": "Mower is stuck",
    "149": "Mower outside perimeter cable",
    "151": "Perimeter cable signal missing",
    "ntfy_blade_life": "Reminder blade life",
}


DAY_MAPPING = {
    0: "monday",
    1: "tuesday",
    2: "wednesday",
    3: "thursday",
    4: "friday",
    5: "saturday",
    6: "sunday",
}



"""API for Bosch API server for Indego lawn mower."""
import logging
import time
import typing

import requests
from requests.auth import HTTPBasicAuth
from requests.exceptions import RequestException, Timeout, TooManyRedirects

from . import __version__
from .const import (
    COMMANDS,
    CONTENT_TYPE,
    CONTENT_TYPE_JSON,
    DEFAULT_BODY,
    DEFAULT_CALENDAR,
    DEFAULT_HEADER,
    Methods,
)
from .indego_base_client import IndegoBaseClient
from .states import Calendar

_LOGGER = logging.getLogger(__name__)


class IndegoClient(IndegoBaseClient):
    """Class for Indego Non-Async Client."""

    def __enter__(self):
        """Enter for with."""
        self.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """Exit for with."""
        pass

    def start(self):
        """Login if not done."""
        if not self._logged_in:
            self.login()

    def delete_alert(self, alert_index: int):
        """Delete the alert with the specified index.

        Args:
            alert_index (int): index of alert to be deleted, should be in range or length of alerts.

        """
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        alert_id = self._get_alert_by_index(alert_index)
        if not alert_id:
            return None
        return self._request(Methods.DELETE, f"alerts/{alert_id}/")

    def delete_all_alerts(self):
        """Delete all the alerts."""
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        if self.alerts_count > 0:
            return [
                self._request(Methods.DELETE, f"alerts/{alert.alert_id}")
                for alert in self.alerts
            ]
        _LOGGER.info("No alerts to delete")
        return None

    def download_map(self, filename: str = None):
        """Download the map.

        Args:
            filename (str, optional): Filename for the map. Defaults to None, can also be filled by the filename set in init.

        """
        if not self.serial:
            return
        if filename:
            self.map_filename = filename
        if not self.map_filename:
            raise ValueError("No map filename defined.")
        lawn_map = self.get(f"alms/{self.serial}/map")
        if lawn_map:
            with open(self.map_filename, "wb") as afp:
                afp.write(lawn_map)

    def put_alert_read(self, alert_index: int):
        """Set the alert to read.

        Args:
            alert_index (int): index of alert to be deleted, should be in range or length of alerts.

        """
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        alert_id = self._get_alert_by_index(alert_index)
        if alert_id:
            return self._request(
                Methods.PUT, f"alerts/{alert_id}", data={"read_status": "read"}
            )

    def put_all_alerts_read(self):
        """Set to read the read_status of all alerts."""
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        if self.alerts_count > 0:
            return [
                self._request(
                    Methods.PUT,
                    f"alerts/{alert.alert_id}",
                    data={"read_status": "read"},
                )
                for alert in self.alerts
            ]
        _LOGGER.info("No alerts to set to read")
        return None

    def put_command(self, command: str):
        """Send a command to the mower.

        Args:
            command (str): command should be one of "mow", "pause", "returnToDock"

        Returns:
            str: either result of the call or 'Wrong Command'

        """
        if command in COMMANDS:
            if not self.serial:
                return None
            return self.put(f"alms/{self.serial}/state", {"state": command})
        raise ValueError("Wrong Command, use one of 'mow', 'pause', 'returnToDock'")

    def put_mow_mode(self, command: typing.Any):
        """Set the mower to mode manual (false-ish) or predictive (true-ish).

        Args:
            command (str/bool): should be str that is bool-ish (true, True, false, False) or a bool.

        Returns:
            str: either result of the call or 'Wrong Command'

        """
        if command in ("true", "false", "True", "False") or isinstance(command, bool):
            if not self.serial:
                return None
            return self.put(f"alms/{self.serial}/predictive", {"enabled": command})
        raise ValueError("Wrong Command, use True or False")

    def put_predictive_cal(self, calendar: dict = DEFAULT_CALENDAR):
        """Set the predictive calendar."""
        try:
            Calendar(**calendar["cals"][0])
        except TypeError as exc:
            raise ValueError("Value for calendar is not valid") from exc
        if not self.serial:
            return
        return self.put(f"alms/{self.serial}/predictive/calendar", calendar)

    def update_alerts(self):
        """Update alerts."""
        self._update_alerts(self.get("alerts"))

    def get_alerts(self):
        """Update alerts and return them."""
        self.update_alerts()
        return self.alerts

    def update_all(self):
        """Update all states."""
        self.update_alerts()
        self.update_calendar()
        self.update_config()
        self.update_generic_data()
        self.update_last_completed_mow()
        self.update_location()
        self.update_network()
        self.update_next_mow()
        self.update_operating_data()
        self.update_predictive_calendar()
        self.update_predictive_schedule()
        self.update_security()
        self.update_setup()
        self.update_state()
        self.update_updates_available()
        self.update_user()

    def update_calendar(self):
        """Update calendar."""
        if not self.serial:
            return
        self._update_calendar(self.get(f"alms/{self.serial}/calendar"))

    def get_calendar(self):
        """Update calendar and return it."""
        self.update_calendar()
        return self.calendar

    def update_config(self):
        """Update config."""
        if not self.serial:
            return
        self._update_config(self.get(f"alms/{self.serial}/config"))

    def get_config(self):
        """Update config and return it."""
        self.update_config()
        return self.config

    def update_generic_data(self):
        """Update generic data."""
        if not self.serial:
            return
        self._update_generic_data(self.get(f"alms/{self.serial}"))

    def get_generic_data(self):
        """Update generic_data and return it."""
        self.update_generic_data()
        return self.generic_data

    def update_last_completed_mow(self):
        """Update last completed mow."""
        if not self.serial:
            return
        self._update_last_completed_mow(
            self.get(f"alms/{self.serial}/predictive/lastcutting")
        )

    def get_last_completed_mow(self):
        """Update last_completed_mow and return it."""
        self.update_last_completed_mow()
        return self.last_completed_mow

    def update_location(self):
        """Update location."""
        if not self.serial:
            return
        self._update_location(self.get(f"alms/{self.serial}/predictive/location"))

    def get_location(self):
        """Update location and return it."""
        self.update_location()
        return self.location

    def update_network(self):
        """Update network."""
        if not self.serial:
            return
        self._update_network(self.get(f"alms/{self.serial}/network"))

    def get_network(self):
        """Update network and return it."""
        self.update_network()
        return self.network

    def update_next_mow(self):
        """Update next mow datetime."""
        if not self.serial:
            return
        self._update_next_mow(self.get(f"alms/{self.serial}/predictive/nextcutting"))

    def get_next_mow(self):
        """Update next_mow and return it."""
        self.update_next_mow()
        return self.next_mow

    def update_operating_data(self):
        """Update operating data."""
        if not self.serial:
            return
        self._update_operating_data(self.get(f"alms/{self.serial}/operatingData"))

    def get_operating_data(self):
        """Update operating_data and return it."""
        self.update_operating_data()
        return self.operating_data

    def update_predictive_calendar(self):
        """Update predictive_calendar."""
        if not self.serial:
            return
        self._update_predictive_calendar(
            self.get(f"alms/{self.serial}/predictive/calendar")
        )

    def get_predictive_calendar(self):
        """Update predictive_calendar and return it."""
        self.update_predictive_calendar()
        return self.predictive_calendar

    def update_predictive_schedule(self):
        """Update predictive_schedule."""
        if not self.serial:
            return
        self._update_predictive_schedule(
            self.get(f"alms/{self.serial}/predictive/schedule")
        )

    def get_predictive_schedule(self):
        """Update predictive_schedule and return it."""
        self.update_predictive_schedule()
        return self.predictive_schedule

    def update_security(self):
        """Update security."""
        if not self.serial:
            return
        self._update_security(self.get(f"alms/{self.serial}/security"))

    def get_security(self):
        """Update security and return it."""
        self.update_security()
        return self.security

    def update_setup(self):
        """Update setup."""
        if not self.serial:
            return
        self._update_setup(self.get(f"alms/{self.serial}/setup"))

    def get_setup(self):
        """Update setup and return it."""
        self.update_setup()
        return self.setup

    def update_state(self, force=False, longpoll=False, longpoll_timeout=120):
        """Update state. Can be both forced and with longpoll.

        Args:
            force (bool, optional): Force the state refresh, wakes up the mower. Defaults to False.
            longpoll (bool, optional): Do a longpoll. Defaults to False.
            longpoll_timeout (int, optional): Timeout of the longpoll. Defaults to 120.

        """
        if not self.serial:
            return
        path = f"alms/{self.serial}/state"
        if longpoll:
            last_state = 0
            if self.state:
                if self.state.state:
                    last_state = self.state.state
            path = f"{path}?longpoll=true&timeout={longpoll_timeout}&last={last_state}"
        if force:
            if longpoll:
                path = f"{path}&forceRefresh=true"
            else:
                path = f"{path}?forceRefresh=true"

        self._update_state(self.get(path, timeout=longpoll_timeout + 30))

    def get_state(self, force=False, longpoll=False, longpoll_timeout=120):
        """Update state. Can be both forced and with longpoll.

        Args:
            force (bool, optional): Force the state refresh, wakes up the mower. Defaults to False.
            longpoll (bool, optional): Do a longpoll. Defaults to False.
            longpoll_timeout (int, optional): Timeout of the longpoll. Defaults to 120.

        """
        self.update_state(force, longpoll, longpoll_timeout)
        return self.state

    def update_updates_available(self):
        """Update updates available."""
        if not self.serial:
            return
        if self._online:
            self._update_updates_available(self.get(f"alms/{self.serial}/updates"))

    def get_updates_available(self):
        """Update updates_available and return it."""
        self.update_updates_available()
        return self.update_available

    def update_user(self):
        """Update users."""
        self._update_user(self.get(f"users/{self._userid}"))

    def get_user(self):
        """Update network and return it."""
        self.update_user()
        return self.user

    def login(self, attempts: int = 0):
        """Login to the api and store the context."""
        response = self._request(
            method=Methods.POST,
            path="authenticate",
            data=DEFAULT_BODY,
            headers=DEFAULT_HEADER,
            auth=HTTPBasicAuth(self._username, self._password),
            timeout=30,
            attempts=attempts,
        )
        self._login(response)
        if response is not None:
            _LOGGER.debug("Logged in")
            if not self._serial:
                list_of_mowers = self.get("alms")
                self._serial = list_of_mowers[0].get("alm_sn")
                _LOGGER.debug("Serial added")
            return True
        return False

    def _request(  # noqa: C901
        self,
        method: Methods,
        path: str,
        data: dict = None,
        headers=None,
        auth=None,
        timeout: int = 30,
        attempts: int = 0,
    ):
        """Send a request and return the response."""
        if attempts >= 3:
            _LOGGER.warning("Three attempts done, waiting 30 seconds.")
            time.sleep(30)
        if attempts >= 5:
            _LOGGER.warning("Five attempts done, please try again manually.")
            return None
        url = f"{self._api_url}{path}"
        if not headers:
            headers = DEFAULT_HEADER.copy()
            headers["x-im-context-id"] = self._contextid
        try:
            response = requests.request(
                method=method.value,
                url=url,
                json=data,
                headers=headers,
                auth=auth,
                timeout=timeout,
            )
            status = response.status_code
            if status == 200:
                if method in (Methods.DELETE, Methods.PATCH, Methods.PUT):
                    return True
                if CONTENT_TYPE_JSON in response.headers[CONTENT_TYPE].split(";"):
                    return response.json()
                return response.content
            if status == 204:
                _LOGGER.info("204: No content in response from server")
                return None
            if status == 400:
                _LOGGER.error("400: Bad Request: won't retry.")
                return None
            if status == 401:
                if path == "authenticate":
                    _LOGGER.info(
                        "401: Unauthorized, credentials are wrong, won't retry"
                    )
                    return None
                _LOGGER.info("401: Unauthorized: logging in again")
                login_result = self.login()
                if login_result:
                    return self._request(
                        method=method,
                        path=path,
                        data=data,
                        timeout=timeout,
                        attempts=attempts + 1,
                    )
                return None
            if status == 403:
                _LOGGER.error("403: Forbidden: won't retry.")
                return None
            if status == 405:
                _LOGGER.error(
                    "405: Method not allowed: Get is used but not allowerd, try a different method for path %s, won't retry.",
                    path,
                )
                return None
            if status == 500:
                _LOGGER.info("500: Internal Server Error")
                return None
            if status == 501:
                _LOGGER.info("501: Not implemented yet")
                return None
            if status == 504:
                if url.find("longpoll=true") > 0:
                    _LOGGER.info("504: longpoll stopped, no updates.")
                    return None
            response.raise_for_status()
        except (Timeout) as exc:
            _LOGGER.error("%s: Timeout on Bosch servers, retrying.", exc)
            return self._request(
                method=method,
                path=path,
                data=data,
                timeout=timeout,
                attempts=attempts + 1,
            )
        except (TooManyRedirects, RequestException) as exc:
            _LOGGER.error("%s: Failed to update Indego status.", exc)
        except Exception as exc:
            _LOGGER.error("Get gave a unhandled error: %s", exc)
        return None

    def get(self, path: str, timeout: int = 30):
        """Send a GET request and return the response as a dict."""
        return self._request(method=Methods.GET, path=path, timeout=timeout)

    def put(self, path: str, data: dict, timeout: int = 30):
        """Send a PUT request and return the response as a dict."""
        return self._request(method=Methods.PUT, path=path, data=data, timeout=timeout)



"""API for Bosch API server for Indego lawn mower."""
import asyncio
import logging
from socket import error as SocketError
from typing import Any

import aiohttp
from aiohttp import (
    ClientOSError,
    ClientResponseError,
    ServerTimeoutError,
    TooManyRedirects,
)
from aiohttp.helpers import BasicAuth
from aiohttp.web_exceptions import HTTPGatewayTimeout

from . import __version__
from .const import (
    COMMANDS,
    CONTENT_TYPE_JSON,
    DEFAULT_BODY,
    DEFAULT_CALENDAR,
    DEFAULT_HEADER,
    DEFAULT_URL,
    Methods,
)
from .indego_base_client import IndegoBaseClient
from .states import Calendar

_LOGGER = logging.getLogger(__name__)


class IndegoAsyncClient(IndegoBaseClient):
    """Class for Indego Async Client."""

    def __init__(
        self,
        username: str,
        password: str,
        serial: str = None,
        map_filename: str = None,
        api_url: str = DEFAULT_URL,
        session: aiohttp.ClientSession = None,
    ):
        """Initialize the Async Client.

        Args:
            username (str): username for Indego Account
            password (str): password for Indego Account
            serial (str): serial number of the mower
            map_filename (str, optional): Filename to store maps in. Defaults to None.
            api_url (str, optional): url for the api, defaults to DEFAULT_URL.

        """
        super().__init__(username, password, serial, map_filename, api_url)
        if session:
            self._session = session
        else:
            self._session = aiohttp.ClientSession(raise_for_status=False)

    async def __aenter__(self):
        """Enter for async with."""
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        """Exit for async with."""
        await self.close()

    async def start(self):
        """Login if not done."""
        if not self._logged_in:
            await self.login()

    async def close(self):
        """Close the aiohttp session."""
        await self._session.close()

    async def delete_alert(self, alert_index: int):
        """Delete the alert with the specified index.

        Args:
            alert_index (int): index of alert to be deleted, should be in range or length of alerts.

        """
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        alert_id = self._get_alert_by_index(alert_index)
        if alert_id:
            return await self._request(Methods.DELETE, f"alerts/{alert_id}/")

    async def delete_all_alerts(self):
        """Delete all the alert."""
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        if self.alerts_count > 0:
            return await asyncio.gather(
                *[
                    self._request(Methods.DELETE, f"alerts/{alert.alert_id}")
                    for alert in self.alerts
                ]
            )
        _LOGGER.info("No alerts to delete")
        return None

    async def download_map(self, filename: str = None):
        """Download the map.

        Args:
            filename (str, optional): Filename for the map. Defaults to None, can also be filled by the filename set in init.

        """
        if not self.serial:
            return
        if filename:
            self.map_filename = filename
        if not self.map_filename:
            raise ValueError("No map filename defined.")
        lawn_map = await self.get(f"alms/{self.serial}/map")
        if lawn_map:
            with open(self.map_filename, "wb") as file:
                file.write(lawn_map)

    async def put_alert_read(self, alert_index: int):
        """Set the alert to read.

        Args:
            alert_index (int): index of alert to be deleted, should be in range or length of alerts.

        """
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        alert_id = self._get_alert_by_index(alert_index)
        if alert_id:
            return await self._request(
                Methods.PUT, f"alerts/{alert_id}", data={"read_status": "read"}
            )

    async def put_all_alerts_read(self):
        """Set to read the read_status of all alerts."""
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        if self.alerts_count > 0:
            return await asyncio.gather(
                *[
                    self._request(
                        Methods.PUT,
                        f"alerts/{alert.alert_id}",
                        data={"read_status": "read"},
                    )
                    for alert in self.alerts
                ]
            )
        _LOGGER.info("No alerts to set to read")
        return None

    async def put_command(self, command: str):
        """Send a command to the mower.

        Args:
            command (str): command should be one of "mow", "pause", "returnToDock"

        Returns:
            str: either result of the call or 'Wrong Command'

        """
        if command in COMMANDS:
            if not self.serial:
                return
            return await self.put(f"alms/{self.serial}/state", {"state": command})
        raise ValueError("Wrong Command, use one of 'mow', 'pause', 'returnToDock'")

    async def put_mow_mode(self, command: Any):
        """Set the mower to mode manual (false-ish) or predictive (true-ish).

        Args:
            command (str/bool): should be str that is bool-ish (true, True, false, False) or a bool.

        Returns:
            str: either result of the call or 'Wrong Command'

        """
        if command in ("true", "false", "True", "False") or isinstance(command, bool):
            if not self.serial:
                return
            return await self.put(
                f"alms/{self.serial}/predictive", {"enabled": command}
            )
        raise ValueError("Wrong Command, use one True or False")

    async def put_predictive_cal(self, calendar: dict = DEFAULT_CALENDAR):
        """Set the predictive calendar."""
        try:
            Calendar(**calendar["cals"][0])
        except TypeError as exc:
            raise ValueError("Value for calendar is not valid") from exc
        if not self.serial:
            return
        return await self.put(f"alms/{self.serial}/predictive/calendar", calendar)

    async def update_alerts(self):
        """Update alerts."""
        self._update_alerts(await self.get("alerts"))

    async def get_alerts(self):
        """Update alerts and return them."""
        await self.update_alerts()
        return self.alerts

    async def update_all(self):
        """Update all states."""
        update_list = [
            self.update_alerts(),
            self.update_calendar(),
            self.update_config(),
            self.update_generic_data(),
            self.update_last_completed_mow(),
            self.update_location(),
            self.update_network(),
            self.update_next_mow(),
            self.update_operating_data(),
            self.update_predictive_calendar(),
            self.update_predictive_schedule(),
            self.update_security(),
            self.update_setup(),
            self.update_state(),
            self.update_updates_available(),
            self.update_user(),
        ]
        results = await asyncio.gather(*update_list, return_exceptions=True)
        for res in results:
            if res:
                _LOGGER.warning(res)

    async def update_calendar(self):
        """Update calendar."""
        if not self.serial:
            return
        self._update_calendar(await self.get(f"alms/{self.serial}/calendar"))

    async def get_calendar(self):
        """Update calendar and return them."""
        await self.update_calendar()
        return self.calendar

    async def update_config(self):
        """Update config."""
        if not self.serial:
            return
        self._update_config(await self.get(f"alms/{self.serial}/config"))

    async def get_config(self):
        """Update config and return it."""
        await self.update_config()
        return self.config

    async def update_generic_data(self):
        """Update generic data."""
        if not self.serial:
            return
        self._update_generic_data(await self.get(f"alms/{self.serial}"))

    async def get_generic_data(self):
        """Update generic_data and return it."""
        await self.update_generic_data()
        return self.generic_data

    async def update_last_completed_mow(self):
        """Update last completed mow."""
        if not self.serial:
            return
        self._update_last_completed_mow(
            await self.get(f"alms/{self.serial}/predictive/lastcutting")
        )

    async def get_last_completed_mow(self):
        """Update last_completed_mow and return it."""
        await self.update_last_completed_mow()
        return self.last_completed_mow

    async def update_location(self):
        """Update location."""
        if not self.serial:
            return
        self._update_location(await self.get(f"alms/{self.serial}/predictive/location"))

    async def get_location(self):
        """Update location and return it."""
        await self.update_location()
        return self.location

    async def update_network(self):
        """Update network."""
        if not self.serial:
            return
        self._update_network(await self.get(f"alms/{self.serial}/network"))

    async def get_network(self):
        """Update network and return it."""
        await self.update_network()
        return self.network

    async def update_next_mow(self):
        """Update next mow datetime."""
        if not self.serial:
            return
        self._update_next_mow(
            await self.get(f"alms/{self.serial}/predictive/nextcutting")
        )

    async def get_next_mow(self):
        """Update next_mow and return it."""
        await self.update_next_mow()
        return self.next_mow

    async def update_operating_data(self):
        """Update operating data."""
        if not self.serial:
            return
        self._update_operating_data(await self.get(f"alms/{self.serial}/operatingData"))

    async def get_operating_data(self):
        """Update operating_data and return it."""
        await self.update_operating_data()
        return self.operating_data

    async def update_predictive_calendar(self):
        """Update predictive_calendar."""
        if not self.serial:
            return
        self._update_predictive_calendar(
            await self.get(f"alms/{self.serial}/predictive/calendar")
        )

    async def get_predictive_calendar(self):
        """Update predictive_calendar and return it."""
        await self.update_predictive_calendar()
        return self.predictive_calendar

    async def update_predictive_schedule(self):
        """Update predictive_schedule."""
        if not self.serial:
            return
        self._update_predictive_schedule(
            await self.get(f"alms/{self.serial}/predictive/schedule")
        )

    async def get_predictive_schedule(self):
        """Update predictive_schedule and return it."""
        await self.update_predictive_schedule()
        return self.predictive_schedule

    async def update_security(self):
        """Update security."""
        if not self.serial:
            return
        self._update_security(await self.get(f"alms/{self.serial}/security"))

    async def get_security(self):
        """Update security and return it."""
        await self.update_security()
        return self.security

    async def update_setup(self):
        """Update setup."""
        if not self.serial:
            return
        self._update_setup(await self.get(f"alms/{self.serial}/setup"))

    async def get_setup(self):
        """Update setup and return it."""
        await self.update_setup()
        return self.setup

    async def update_state(self, force=False, longpoll=False, longpoll_timeout=120):
        """Update state. Can be both forced and with longpoll.

        Args:
            force (bool, optional): Force the state refresh, wakes up the mower. Defaults to False.
            longpoll (bool, optional): Do a longpoll. Defaults to False.
            longpoll_timeout (int, optional): Timeout of the longpoll. Defaults to 120, maximum is 300.

        Raises:
            ValueError: when the longpoll timeout is longer then 300 seconds.

        """
        if not self.serial:
            return
        path = f"alms/{self.serial}/state"
        if longpoll:
            if longpoll_timeout > 300:
                raise ValueError(
                    "Longpoll timeout must be less than or equal 300 seconds."
                )
            last_state = 0
            if self.state:
                if self.state.state:
                    last_state = self.state.state
            path = f"{path}?longpoll=true&timeout={longpoll_timeout}&last={last_state}"
        if force:
            if longpoll:
                path = f"{path}&forceRefresh=true"
            else:
                path = f"{path}?forceRefresh=true"

        self._update_state(await self.get(path, timeout=longpoll_timeout + 30))

    async def get_state(self, force=False, longpoll=False, longpoll_timeout=120):
        """Update state and return it.

        Args:
            force (bool, optional): Force the state refresh, wakes up the mower. Defaults to False.
            longpoll (bool, optional): Do a longpoll. Defaults to False.
            longpoll_timeout (int, optional): Timeout of the longpoll. Defaults to 120, maximum is 300.

        Raises:
            ValueError: when the longpoll timeout is longer then 300 seconds.

        """
        await self.update_state(force, longpoll, longpoll_timeout)
        return self.state

    async def update_updates_available(self):
        """Update updates available."""
        if not self.serial:
            return
        if self._online:
            self._update_updates_available(
                await self.get(f"alms/{self.serial}/updates")
            )

    async def get_updates_available(self):
        """Update updates_available and return it."""
        await self.update_updates_available()
        return self.update_available

    async def update_user(self):
        """Update users."""
        self._update_user(await self.get(f"users/{self._userid}"))

    async def get_user(self):
        """Update user and return it."""
        await self.update_user()
        return self.user

    async def login(self, attempts: int = 0):
        """Login to the api and store the context."""
        response = await self._request(
            method=Methods.POST,
            path="authenticate",
            data=DEFAULT_BODY,
            headers=DEFAULT_HEADER,
            auth=BasicAuth(self._username, self._password),
            timeout=30,
            attempts=attempts,
        )
        self._login(response)
        if response is not None:
            _LOGGER.debug("Logged in")
            if not self._serial:
                list_of_mowers = await self.get("alms")
                self._serial = list_of_mowers[0].get("alm_sn")
                _LOGGER.debug("Serial added")
            return True
        return False

    async def _request(  # noqa: C901
        self,
        method: Methods,
        path: str,
        data: dict = None,
        headers: dict = None,
        auth: BasicAuth = None,
        timeout: int = 30,
        attempts: int = 0,
    ):
        """Request implemented by the subclasses either synchronously or asynchronously.

        Args:
            method (Methods): HTTP method to be executed.
            path (str): url to call on top of base_url.
            data (dict, optional): if applicable, data to be sent, defaults to None.
            headers (dict, optional): headers to be included, defaults to None, which should be filled by the method.
            auth (BasicAuth or HTTPBasicAuth, optional): login specific attribute, defaults to None.
            timeout (int, optional): Timeout for the api call. Defaults to 30.
            attempts (int, optional): Number to keep track of retries, after three starts delaying, after five quites.

        """
        if 3 <= attempts < 5:
            _LOGGER.info("Three or four attempts done, waiting 30 seconds")
            await asyncio.sleep(30)
        if attempts == 5:
            _LOGGER.warning("Five attempts done, please try again later")
            return None
        url = f"{self._api_url}{path}"
        if not headers:
            headers = DEFAULT_HEADER.copy()
            headers["x-im-context-id"] = self._contextid
        _LOGGER.debug("Sending %s to %s", method.value, url)
        try:
            async with self._session.request(
                method=method.value,
                url=url,
                json=data if data else DEFAULT_BODY,
                headers=headers,
                auth=auth,
                timeout=timeout,
            ) as response:
                status = response.status
                _LOGGER.debug("status: %s", status)
                if status == 200:
                    if response.content_type == CONTENT_TYPE_JSON:
                        resp = await response.json()
                        _LOGGER.debug("Response: %s", resp)
                        return resp  # await response.json()
                    return await response.content.read()
                if status == 204:
                    _LOGGER.debug("204: No content in response from server")
                    return None
                if status == 400:
                    _LOGGER.warning(
                        "400: Bad Request: won't retry. Message: %s",
                        (await response.content.read()).decode("UTF-8"),
                    )
                    return None
                if status == 401:
                    if path == "authenticate":
                        _LOGGER.info(
                            "401: Unauthorized, credentials are wrong, won't retry"
                        )
                        return None
                    _LOGGER.info("401: Unauthorized: logging in again")
                    login_result = await self.login()
                    if login_result:
                        return await self._request(
                            method=method,
                            path=path,
                            data=data,
                            timeout=timeout,
                            attempts=attempts + 1,
                        )
                    return None
                if status == 403:
                    _LOGGER.error("403: Forbidden: won't retry")
                    return None
                if status == 405:
                    _LOGGER.error(
                        "405: Method not allowed: %s is used but not allowed, try a different method for path %s, won't retry",
                        method,
                        path,
                    )
                    return None
                if status == 500:
                    _LOGGER.debug("500: Internal Server Error")
                    return None
                if status == 501:
                    _LOGGER.debug("501: Not implemented yet")
                    return None
                if status == 504:
                    if url.find("longpoll=true") > 0:
                        _LOGGER.debug("504: longpoll stopped, no updates")
                        return None
                response.raise_for_status()
        except (asyncio.TimeoutError, ServerTimeoutError, HTTPGatewayTimeout) as exc:
            _LOGGER.info("%s: Timeout on Bosch servers, retrying", exc)
            return await self._request(
                method=method,
                path=path,
                data=data,
                timeout=timeout,
                attempts=attempts + 1,
            )
        except ClientOSError as exc:
            _LOGGER.debug("%s: Failed to update Indego status, longpoll timeout", exc)
            return None
        except (TooManyRedirects, ClientResponseError, SocketError) as exc:
            _LOGGER.error("%s: Failed %s to Indego, won't retry", exc, method.value)
            return None
        except asyncio.CancelledError:
            _LOGGER.debug("Task cancelled by task runner")
            return None
        except Exception as exc:
            _LOGGER.error("Request to %s gave a unhandled error: %s", url, exc)
            return None

    async def get(self, path: str, timeout: int = 30):
        """Get implemented by the subclasses either synchronously or asynchronously.

        Args:
            path (str): url to call on top of base_url
            timeout (int, optional): Timeout for the api call. Defaults to 30.

        """
        return await self._request(method=Methods.GET, path=path, timeout=timeout)

    async def put(self, path: str, data: dict, timeout: int = 30):
        """Put implemented by the subclasses either synchronously or asynchronously.

        Args:
            path (str): url to call on top of base_url
            data (dict): data to put
            timeout (int, optional): Timeout for the api call. Defaults to 30.

        """
        return await self._request(
            method=Methods.PUT, path=path, data=data, timeout=timeout
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import datetime
import json
from pyIndego import IndegoClient

def main(config):
    with IndegoClient(**config) as indego:
        indego.update_predictive_schedule()
        
        print("Times where SmartMowing is planing to mow the lawn:")
        
        for i in range(datetime.now().weekday(), datetime.now().weekday()+7):
            for j in range(len(indego.predictive_schedule.schedule_days)):
                if (indego.predictive_schedule.schedule_days[j].day == (i % 7)):
                    print("\t{}".format(indego.predictive_schedule.schedule_days[j].day_name))
                    for k in range(len(indego.predictive_schedule.schedule_days[j].slots)):
                        print('\t\t{:%H:%M} - {:%H:%M}'.format(indego.predictive_schedule.schedule_days[j].slots[k].start, indego.predictive_schedule.schedule_days[j].slots[k].end))
        
        print("Times that are excluded for mowing from SmartMowing:")
        
        for i in range(datetime.now().weekday(), datetime.now().weekday()+7):
            for j in range(len(indego.predictive_schedule.exclusion_days)):
                if (indego.predictive_schedule.exclusion_days[j].day == (i % 7)):
                    print("\t{}".format(indego.predictive_schedule.exclusion_days[j].day_name))
                    for k in range(len(indego.predictive_schedule.exclusion_days[j].slots)):
                        print('\t\t{:%H:%M} - {:%H:%M} {}'.format(indego.predictive_schedule.exclusion_days[j].slots[k].start, indego.predictive_schedule.exclusion_days[j].slots[k].end, indego.predictive_schedule.exclusion_days[j].slots[k].Attr))

if __name__ == "__main__":
    with open("config.json", "r") as config_file:
        config = json.load(config_file)
    main(config)



"""Base class for indego."""
import logging
from abc import ABC, abstractmethod
from typing import Any

import pytz

from .const import (
    DEFAULT_CALENDAR,
    DEFAULT_LOOKUP_VALUE,
    DEFAULT_URL,
    MOWER_STATE_DESCRIPTION,
    MOWER_STATE_DESCRIPTION_DETAIL,
    Methods,
)
from .helpers import convert_bosch_datetime, generate_update
from .states import (
    Alert,
    Calendar,
    Config,
    GenericData,
    Location,
    Network,
    OperatingData,
    PredictiveSchedule,
    Security,
    Setup,
    State,
    User,
)

_LOGGER = logging.getLogger(__name__)


class IndegoBaseClient(ABC):
    """Indego base client class."""

    def __init__(
        self,
        username: str,
        password: str,
        serial: str = None,
        map_filename: str = None,
        api_url: str = DEFAULT_URL,
    ):
        """Abstract class for the Indego Clent, only use the Indego Client or Indego Async Client.

        Args:
            username (str): username for Indego Account
            password (str): password for Indego Account
            serial (str): serial number of the mower
            map_filename (str, optional): Filename to store maps in. Defaults to None.
            api_url (str, optional): url for the api, defaults to DEFAULT_URL.

        """
        self._username = username
        self._password = password
        self._serial = serial
        self.map_filename = map_filename
        self._api_url = api_url
        self._logged_in = False
        self._online = False
        self._contextid = ""
        self._userid = None

        self.alerts = []
        self._alerts_loaded = False
        self.calendar = None
        self.config = None
        self.generic_data = None
        self.last_completed_mow = None
        self.location = None
        self.network = None
        self.next_mow = None
        self.operating_data = None
        self.predictive_calendar = None
        self.predictive_schedule = None
        self.security = None
        self.state = None
        self.setup = None
        self.runtime = None
        self.update_available = False
        self.user = None

    # Properties
    @property
    def serial(self):
        """Return the serial number of the mower."""
        if self._serial:
            return self._serial
        _LOGGER.warning("Serial not yet set, please login first")
        return None

    @property
    def alerts_count(self):
        """Return the count of alerts."""
        if self.alerts:
            return len(self.alerts)
        return 0

    @property
    def state_description(self):
        """Return the description of the state."""
        if self.state:
            return MOWER_STATE_DESCRIPTION.get(self.state.state, DEFAULT_LOOKUP_VALUE)
        _LOGGER.warning("Please call update_state before calling this property")
        return None

    @property
    def state_description_detail(self):
        """Return the description detail of the state."""
        if self.state:
            return MOWER_STATE_DESCRIPTION_DETAIL.get(
                self.state.state, DEFAULT_LOOKUP_VALUE
            )
        _LOGGER.warning("Please call update_state before calling this property")
        return None

    @property
    def next_mows(self):
        """Return the next mows from the calendar without a timezone."""
        if self.calendar:
            return [
                slot.dt for day in self.calendar.days for slot in day.slots if slot.dt
            ]
        _LOGGER.warning("Please call update_calendar before calling this property")
        return None

    @property
    def next_mows_with_tz(self):
        """Return the next mows from the calendar with timezone from location."""
        if self.location and self.calendar:
            return [
                slot.dt.astimezone(pytz.timezone(self.location.timezone))
                for day in self.calendar.days
                for slot in day.slots
                if slot.dt
            ]
        if not self.location:
            _LOGGER.warning("Please call update_location before calling this property")
        if not self.calendar:
            _LOGGER.warning("Please call update_calendar before calling this property")
        return None

    # Methods
    @abstractmethod
    def delete_alert(self, alert_index: int):
        """Delete the alert with the specified index."""

    @abstractmethod
    def delete_all_alerts(self):
        """Delete all the alerts."""

    @abstractmethod
    def download_map(self, filename=None):
        """Download the map."""

    @abstractmethod
    def put_alert_read(self, alert_index: int):
        """Set to read the read_status of the alert with the specified index."""

    @abstractmethod
    def put_all_alerts_read(self):
        """Set to read the read_status of all alerts."""

    @abstractmethod
    def put_command(self, command: str):
        """Send a command to the mower."""

    @abstractmethod
    def put_mow_mode(self, command: Any):
        """Set the mower to mode manual (false-ish) or predictive (true-ish)."""

    @abstractmethod
    def put_predictive_cal(self, calendar: dict = DEFAULT_CALENDAR):
        """Set the predictive calendar."""

    @abstractmethod
    def update_alerts(self):
        """Update alerts."""

    @abstractmethod
    def get_alerts(self):
        """Update alerts and return them."""

    def _update_alerts(self, new):
        """Update alerts."""
        self._alerts_loaded = True
        if new:
            self.alerts = [Alert(**a) for a in new]
        else:
            self.alerts = []

    @abstractmethod
    def update_all(self):
        """Update all states."""

    @abstractmethod
    def update_calendar(self):
        """Update calendar."""

    @abstractmethod
    def get_calendar(self):
        """Update calendar and return it."""

    def _update_calendar(self, new):
        """Update calendar."""
        if new:
            self.calendar = generate_update(self.calendar, new["cals"][0], Calendar)

    @abstractmethod
    def update_config(self):
        """Update config."""

    @abstractmethod
    def get_config(self):
        """Update config and return it."""

    def _update_config(self, new):
        """Update config."""
        if new:
            self.config = generate_update(self.config, new, Config)

    @abstractmethod
    def update_generic_data(self):
        """Update generic data."""

    @abstractmethod
    def get_generic_data(self):
        """Update generic_data and return it."""

    def _update_generic_data(self, new):
        """Update generic data."""
        if new:
            self.generic_data = generate_update(self.generic_data, new, GenericData)
            self._update_battery_percentage_adjusted()

    @abstractmethod
    def update_last_completed_mow(self):
        """Update last completed mow."""

    @abstractmethod
    def get_last_completed_mow(self):
        """Update last_completed_mow and return it."""

    def _update_last_completed_mow(self, new):
        """Update last completed mow."""
        if new:
            self.last_completed_mow = convert_bosch_datetime(new["last_mowed"])

    @abstractmethod
    def update_location(self):
        """Update location."""

    @abstractmethod
    def get_location(self):
        """Update location and return it."""

    def _update_location(self, new):
        """Update location."""
        if new:
            self.location = generate_update(self.location, new, Location)

    @abstractmethod
    def update_network(self):
        """Update network."""

    @abstractmethod
    def get_network(self):
        """Update network and return it."""

    def _update_network(self, new):
        """Update network."""
        if new:
            self.network = generate_update(self.network, new, Network)

    @abstractmethod
    def update_next_mow(self):
        """Update next mow datetime."""

    @abstractmethod
    def get_next_mow(self):
        """Update next_mow and return it."""

    def _update_next_mow(self, new):
        """Update next mow datetime."""
        if new:
            self.next_mow = convert_bosch_datetime(new["mow_next"])

    @abstractmethod
    def update_operating_data(self):
        """Update operating data."""

    @abstractmethod
    def get_operating_data(self):
        """Update operating_data and return it."""

    def _update_operating_data(self, new):
        """Update operating data."""
        if new:
            self.operating_data = generate_update(
                self.operating_data, new, OperatingData
            )
            self._update_battery_percentage_adjusted()
            self._online = True
        else:
            self._online = False

    @abstractmethod
    def update_predictive_calendar(self):
        """Update predictive_calendar."""

    @abstractmethod
    def get_predictive_calendar(self):
        """Update predictive_calendar and return it."""

    def _update_predictive_calendar(self, new):
        """Update predictive_calendar."""
        if new:
            self.predictive_calendar = generate_update(
                self.predictive_calendar, new["cals"][0], Calendar
            )

    @abstractmethod
    def update_predictive_schedule(self):
        """Update predictive schedule."""

    @abstractmethod
    def get_predictive_schedule(self):
        """Update predictive_schedule and return it."""

    def _update_predictive_schedule(self, new):
        """Update predictive schedule."""
        if new:
            self.predictive_schedule = generate_update(
                self.predictive_schedule, new, PredictiveSchedule
            )

    @abstractmethod
    def update_security(self):
        """Update security."""

    @abstractmethod
    def get_security(self):
        """Update security and return it."""

    def _update_security(self, new):
        """Update security."""
        if new:
            self.security = generate_update(self.security, new, Security)

    @abstractmethod
    def update_setup(self):
        """Update setup."""

    @abstractmethod
    def get_setup(self):
        """Update setup and return it."""

    def _update_setup(self, new):
        """Update setup."""
        if new:
            self.setup = generate_update(self.setup, new, Setup)

    @abstractmethod
    def update_state(self, force=False, longpoll=False, longpoll_timeout=120):
        """Update state. Can be both forced and with longpoll."""

    @abstractmethod
    def get_state(self, force=False, longpoll=False, longpoll_timeout=120):
        """Update state and return it."""

    def _update_state(self, new):
        """Update state."""
        if new:
            self.state = generate_update(self.state, new, State)

    @abstractmethod
    def update_updates_available(self):
        """Update updates available."""

    @abstractmethod
    def get_updates_available(self):
        """Update updates_available and return it."""

    def _update_updates_available(self, new):
        """Update updates available."""
        if new:
            self.update_available = bool(new["available"])

    @abstractmethod
    def update_user(self):
        """Update users."""

    @abstractmethod
    def get_user(self):
        """Update user and return it."""

    def _update_user(self, new):
        """Update users."""
        if new:
            self.user = generate_update(self.user, new, User)

    @abstractmethod
    def login(self, attempts: int = 0):
        """Login to the Indego API."""

    def _login(self, login):
        """Login to the Indego API."""
        if login:
            self._contextid = login["contextId"]
            self._userid = login["userId"]
            self._logged_in = True
        else:
            self._logged_in = False

    @abstractmethod
    def _request(
        self,
        method: Methods,
        path: str,
        data: dict = None,
        headers: dict = None,
        auth: Any = None,
        timeout: int = 30,
        attempts: int = 0,
    ):
        """Request implemented by the subclasses either synchronously or asynchronously."""

    @abstractmethod
    def get(self, path: str, timeout: int):
        """Get implemented by the subclasses either synchronously or asynchronously."""

    @abstractmethod
    def put(self, path: str, data: dict, timeout: int):
        """Put implemented by the subclasses either synchronously or asynchronously."""

    # internal methods
    def _get_alert_by_index(self, alert_index: int) -> int:
        """Return the alert_id based on index."""
        if not self._alerts_loaded:
            raise ValueError("Alerts not loaded, please run update_alerts first.")
        if self.alerts_count == 0:
            _LOGGER.info("No alerts to get")
            return None
        try:
            return self.alerts[alert_index].alert_id
        except IndexError as exc:
            raise IndexError(
                f"Wrong index for the alert, there are {self.alerts_count} alerts, so the highest index is: {self.alerts_count - 1}, supplied was {alert_index}"
            ) from exc

    def _update_battery_percentage_adjusted(self):
        """Update the battery percentage adjusted field, relies on generic and operating data populated."""
        if self.generic_data and self.operating_data:
            self.operating_data.battery.update_percent_adjusted(
                self.generic_data.model_voltage
            )

    def __repr__(self):
        """Create a string representing the mower."""
        str1 = (
            f"{self.generic_data.model_description} ({self.generic_data.alm_sn})"
            if self.generic_data
            else f"Indego mower"
        )
        str2 = f" owned by {self.user.display_name}." if self.user else f"."
        str3 = f" {self.generic_data}, " if self.generic_data else ""
        str4 = f" {self.state}, " if self.state else ""
        str5 = f" {self.operating_data}, " if self.operating_data else ""
        str6 = f", last mowed: {self.last_completed_mow}, next mow: {self.next_mow}, {self.location}, {self.network}, {self.alerts}, map filename: {self.map_filename}, {self.runtime}"
        str7 = f"{self.operating_data.battery} " if self.operating_data else ""
        str8 = f"update available: {self.update_available}, State Descr: {self.state_description}."
        return f"{str1}{str2}{str3}{str4}{str5}{str6}{str7}{str8}"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
from datetime import datetime, timezone, timedelta
from pyIndego import IndegoClient

def main(config):
    with IndegoClient(**config) as indego:
        
        indego.update_next_mow()
        print("Next mowing:", indego.next_mow)
        
        nowDate = datetime.now(timezone.utc)
        if (indego.next_mow - nowDate) < timedelta(hours=2, minutes=30):
            print("Less than two and a half hours before mowing.")

if __name__ == "__main__":
    with open("config.json", "r") as config_file:
        config = json.load(config_file)
    main(config)



"""Helper class for Indego."""
import logging
from dataclasses import dataclass, is_dataclass, replace
from datetime import datetime
from typing import Any

_LOGGER = logging.getLogger(__name__)


def nested_dataclass(*args, **kwargs):  # noqa: D202
    """Wrap a nested dataclass object."""

    def wrapper(cls):
        cls = dataclass(cls, **kwargs)
        original_init = cls.__init__

        def __init__(self, *args, **kwargs):
            for name, value in kwargs.items():
                field_type = cls.__annotations__.get(name, None)
                if hasattr(field_type, "__args__"):
                    inner_type = field_type.__args__[0]
                    if is_dataclass(inner_type):
                        new_obj = [inner_type(**dict_) for dict_ in value]
                        kwargs[name] = new_obj
                else:
                    if is_dataclass(field_type) and isinstance(value, dict):
                        new_obj = field_type(**value)
                        kwargs[name] = new_obj

            original_init(self, *args, **kwargs)

        cls.__init__ = __init__
        return cls

    return wrapper(args[0]) if args else wrapper


def convert_bosch_datetime(dt: Any = None) -> datetime:
    """Create a datetime object from the string (or give back the datetime object) from Bosch. Checks if a valid number of milliseconds is sent."""
    if dt:
        if isinstance(dt, str):
            if dt.find(".") > 0:
                return datetime.strptime(dt, "%Y-%m-%dT%H:%M:%S.%f%z")
            return datetime.strptime(dt, "%Y-%m-%dT%H:%M:%S%z")
        if isinstance(dt, datetime):
            return dt
    return None


def generate_update(field: Any, new: dict, new_class: Any):
    """Update a field to the new value, or instantiated the class and return the updated or new.

    Args:
        field (None|State Class): current value of the to be updated field.
        new (dict): new values coming back from the api.
        new_class (State Class): Class to instantiate the value with if necessary.

    Returns:
        (new_class): new value of the type that was passed as the new_class.

    """
    if field:
        return replace(field, **new)
    return new_class(**new)



"""Classes for states of pyIndego."""
import logging
from dataclasses import dataclass, field
from datetime import date, datetime, time, timedelta
from typing import List

from .const import (
    ALERT_ERROR_CODE,
    DAY_MAPPING,
    DEFAULT_LOOKUP_VALUE,
    MOWER_MODEL_DESCRIPTION,
    MOWING_MODE_DESCRIPTION,
)
from .helpers import convert_bosch_datetime, nested_dataclass

_LOGGER = logging.getLogger(__name__)


@dataclass
class Alert:
    """Alert class."""

    alm_sn: str = field(repr=False, default=None)
    alert_id: str = None
    error_code: str = None
    headline: str = None
    date: datetime = None
    message: str = None
    read_status: str = None
    flag: str = None
    push: bool = None
    alert_description: str = None

    def __post_init__(self):
        """Set alert description."""
        self.alert_description = ALERT_ERROR_CODE.get(
            self.error_code, DEFAULT_LOOKUP_VALUE
        )
        self.date = convert_bosch_datetime(self.date)


@dataclass
class ModelVoltage:
    """Model voltage Class."""

    min: int = None
    max: int = None


MOWER_MODEL_VOLTAGE = {
    "3600HA2300": ModelVoltage(min=285, max=369),   # Indego 1000
    "3600HA2301": ModelVoltage(min=285, max=369),   # Indego 1200
    "3600HA2302": ModelVoltage(min=285, max=369),   # Indego 1100
    "3600HA2303": ModelVoltage(min=285, max=369),   # Indego 13C
    "3600HA2304": ModelVoltage(min=285, max=369),   # Indego 10C
    "3600HB0100": ModelVoltage(min=0, max=100),     # Indego 350
    "3600HB0101": ModelVoltage(min=0, max=100),     # Indego 400
    "3600HB0102": ModelVoltage(min=0, max=100),     # Indego S+ 350
    "3600HB0103": ModelVoltage(min=0, max=100),     # Indego S+ 400
    "3600HB0105": ModelVoltage(min=0, max=100),     # Indego S+ 350
    "3600HB0106": ModelVoltage(min=0, max=100),     # Indego S+ 400
    "3600HB0302": ModelVoltage(min=0, max=100),     # Indego S+ 500
    "3600HB0301": ModelVoltage(min=0, max=100),     # Indego M+ 700
    #    '3600HB0xxx': {'min': '0','max': '100'}   # Indego M+ 700
}


@dataclass
class Battery:
    """Battery Class."""

    percent: int = None
    voltage: float = None
    cycles: int = None
    discharge: float = None
    ambient_temp: int = None
    battery_temp: int = None
    percent_adjusted: int = None

    def update_percent_adjusted(self, voltage: ModelVoltage):
        """Set percent adjusted."""
        if self.percent:
            self.percent_adjusted = round(
                (int(self.percent) - voltage.min) / ((voltage.max - voltage.min) / 100)
            )


@dataclass
class CalendarSlot:
    """Class for CalendarSlots."""

    En: bool = None
    StHr: int = None
    StMin: int = None
    EnHr: int = None
    EnMin: int = None
    Attr: str = None
    start: time = None
    end: time = None
    dt: datetime = None

    def __post_init__(self):
        """Convert start and end in time format."""
        if self.StHr is not None and self.StMin is not None:
            self.start = time(self.StHr, self.StMin)
        if self.EnHr is not None and self.EnMin is not None:
            self.end = time(self.EnHr, self.EnMin)


@nested_dataclass
class CalendarDay:
    """Class for CalendarDays."""

    day: int = None
    day_name: str = None
    slots: List[CalendarSlot] = field(default_factory=lambda: [CalendarSlot])

    def __post_init__(self):
        """Update the dayname."""
        if self.day is not None:
            self.day_name = DAY_MAPPING[self.day]
        if self.slots:
            for slot in self.slots:
                if slot.En:
                    today = date.today().weekday()
                    date_offset = timedelta(
                        days=self.day - today, hours=slot.StHr, minutes=slot.StMin
                    )
                    new_dt = (
                        datetime.now().replace(
                            hour=0, minute=0, second=0, microsecond=0
                        )
                        + date_offset
                    )
                    if new_dt.date() < date.today():
                        new_dt = new_dt + timedelta(days=7)
                    slot.dt = new_dt


@nested_dataclass
class Calendar:
    """Class for Calendar."""

    cal: int = None
    days: List[CalendarDay] = field(default_factory=lambda: [CalendarDay])


@nested_dataclass
class PredictiveSchedule:
    """Class for PredictiveSchedule."""

    schedule_days: List[CalendarDay] = field(default_factory=lambda: [CalendarDay])
    exclusion_days: List[CalendarDay] = field(default_factory=lambda: [CalendarDay])


@nested_dataclass
class GenericData:
    """Generic Data Class."""

    alm_name: str = None
    alm_sn: str = None
    service_counter: int = None
    needs_service: bool = None
    alm_mode: str = None
    bareToolnumber: str = None
    alm_firmware_version: str = None
    model_description: str = None
    model_voltage: ModelVoltage = field(default_factory=ModelVoltage)
    mowing_mode_description: str = None

    def __post_init__(self):
        """Set model description, voltage, mode description."""
        self.model_description = MOWER_MODEL_DESCRIPTION.get(
            self.bareToolnumber, DEFAULT_LOOKUP_VALUE
        )
        self.model_voltage = MOWER_MODEL_VOLTAGE.get(
            self.bareToolnumber, ModelVoltage()
        )
        self.mowing_mode_description = MOWING_MODE_DESCRIPTION.get(
            self.alm_mode, DEFAULT_LOOKUP_VALUE
        )


@dataclass
class Location:
    """Location Class."""

    latitude: float = None
    longitude: float = None
    timezone: str = None


@dataclass
class Network:
    """Network Class."""

    mcc: int = None
    mnc: int = None
    rssi: int = None
    currMode: str = None
    configMode: str = None
    steeredRssi: int = None
    networkCount: int = None
    networks: List[int] = None


@dataclass
class Config:
    """Config Class."""

    region: int = None
    language: int = None
    border_cut: int = None
    is_pin_set: bool = None
    wire_id: int = None
    bump_sensitivity: int = None
    alarm_mode: bool = None


@dataclass
class Setup:
    """Setup Class."""

    hasOwner: bool = None
    hasPin: bool = None
    hasMap: bool = None
    hasAutoCal: bool = None
    hasIntegrityCheckPassed: bool = None


@dataclass
class Security:
    """Security Class."""

    enabled: bool = None
    autolock: bool = None


@dataclass
class RuntimeDetail:
    """Runtime Details Class."""

    operate: int = None
    charge: int = None
    cut: int = field(init=False, default=None)

    def update_cut(self):
        """Update cut."""
        self.cut = round(self.operate - self.charge)


@nested_dataclass
class Runtime:  # pylint: disable=no-member,assigning-non-slot
    """Runtime Class."""

    total: RuntimeDetail = field(default_factory=RuntimeDetail)
    session: RuntimeDetail = field(default_factory=RuntimeDetail)

    def __post_init__(self):
        """Set cuts and calc totals."""
        if self.total.charge:
            self.total.charge = round(self.total.charge / 100)
        if self.total.operate:
            self.total.operate = round(self.total.operate / 100)
        if self.total.charge:
            self.total.update_cut()
        if self.session.charge:
            self.session.update_cut()
        else:
            self.session.cut = 0


@dataclass
class Garden:
    """Garden Class."""

    id: int = None
    name: int = None
    signal_id: int = None
    size: int = None
    inner_bounds: int = None
    cuts: int = None
    runtime: int = None
    charge: int = None
    bumps: int = None
    stops: int = None
    last_mow: int = None
    map_cell_size: int = None


@nested_dataclass
class OperatingData:
    """Operating Data Class."""

    hmiKeys: str = None
    battery: Battery = field(default_factory=Battery)
    garden: Garden = field(default_factory=Garden)
    runtime: Runtime = field(default_factory=Runtime)


@nested_dataclass
class State:
    """State Class."""

    state: int = None
    map_update_available: bool = None
    mowed: int = None
    mowmode: int = None
    error: int = None
    xPos: int = None
    yPos: int = None
    charge: int = None
    operate: int = None
    runtime: Runtime = field(default_factory=Runtime)
    mapsvgcache_ts: int = None
    svg_xPos: int = None
    svg_yPos: int = None
    config_change: bool = None
    mow_trig: bool = None


@dataclass
class User:
    """User Class."""

    email: str = None
    display_name: str = None
    language: str = None
    country: str = None
    optIn: bool = None
    optInApp: bool = None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
from pyIndego import IndegoClient

def country_operator(mcc, mnc):
    if mcc == 262:
        country =  "Germany"
        if mnc == 1:
            operator = "Telekom Deutschland"
        elif mnc == 2:
            operator = "Vodafone"
        elif mnc == 3:
            operator = "Telefónica"
        else:
            operator = None
    elif mcc == 232:
        country =  "Austria"
        if mnc == 1:
            operator = "A1 Telekom Austria"
        elif mnc == 2:
            operator = "Magenta Telekom"
        elif mnc == 5:
            operator = "Hutchison Drei Austria"
    elif mcc == 228:
        country =  "Switzerland"
        if mnc == 1:
            operator = "Swisscom"
        elif mnc == 2:
            operator = "Sunrise Communications"
        elif mnc == 3:
            operator = "Salt Mobile SA"
        else:
            operator = None
    else:
        country = None
        operator = None
    return (country, operator)


def main(config):
    with IndegoClient(**config) as indego:
        indego.update_network()
        
        (country, operator) = country_operator(indego.network.mcc, indego.network.mnc)
        if country is not None:
            print("Country is:", country)
            if operator is not None:
                print("Operator is:", operator)
            else:
                print("Operator is unknown")
        else:
            print("Country and operator are unknown")
        
        print("Signal strength (rssi):", indego.network.rssi)
        
        print("Available Networks:")
        for i in range(indego.network.networkCount):
            (country, operator) = country_operator(int(str(indego.network.networks[i])[:3]), int(str(indego.network.networks[i])[3:5]))
            if (country is not None) and (operator is not None):
                print("\t", country, ":", operator)
            else:
                print("\tmcc =", str(indego.network.networks[i])[:3], ": mnc =", str(indego.network.networks[i])[3:5])
        
if __name__ == "__main__":
    with open("config.json", "r") as config_file:
        config = json.load(config_file)
    main(config)



"""Setup for pyIndego."""
from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pyIndego",
    version="2.0.28",
    author="jm-73",
    author_email="jens@myretyr.se",
    description="API for Bosch Indego mower",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jm-73/pyIndego",
    packages=find_packages("."),
    install_requires=["requests", "aiohttp", "pytz"],
    extras_require={"testing": ["pytest", "pytest-aiohttp", "pytest-cov", "mock", "pytest-asyncio"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



"""test aio client."""
import asyncio
import inspect
import json
import logging
import time
import datetime

import aiohttp

from pyIndego import IndegoAsyncClient
from pyIndego.const import Methods

logging.basicConfig(level=logging.DEBUG)
_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)


async def main(config):
    """Test class of test aio."""
    async with IndegoAsyncClient(**config) as indego:
        #Update and cache all calls from API
        #await indego.update_all()
        
        # await indego.update_alerts()
        # print("alerts: ", indego.alerts)
        ###### FUNCTION! print("alert count: ", indego.alerts_count)

        # await indego.update_calendar()
        # print("calendar: ", indego.calendar)
        ###### FUNCTION! print("next mow: ", indego.next_mows)

        # await indego.update_config()
        # print("config: ", indego.config)

        # await indego.update_generic_data()
        # print("generic data: ", indego.generic_data)

        # await indego.update_last_completed_mow()
        # print("last completed mow: ", indego.last_completed_mow)

        # await indego.update_location()
        # print("location: ", indego.location)

        # await indego.update_network()
        # print("network: ", indego.network)

        # await indego.update_next_mow()
        # print("next mow: ", indego.next_mow)

        # await indego.update_operating_data()
        # print("operating data: ", indego.operating_data)

        # await indego.update_predictive_calendar()
        # print("predictive calendar: ", indego.predictive_calendar)
# 
        # await indego.update_predictive_schedule()
        # print("predictive schedule: ", indego.predictive_schedule)

        # await indego.update_security()
        # print("security: ", indego.security)

        # await indego.update_setup()
        # print("setup: ", indego.setup)
        jens = False
        while not (jens == True):
            await indego.update_state()
            #await indego.update_state(force=True)
            # print("state: ", indego.state)
            # print("state description: ", indego.state_description)
            # print("state description detail: ", indego.state_description_detail)
            # print("serial: ", indego.serial)
            #print("xPos: ", indego.state.xPos)
            #print("yPos: ", indego.state.yPos)
            
            f = open('percentage.log', 'a')
            
            from time import gmtime, strftime
            logtime = strftime("%H:%M", gmtime())
            print(logtime , end = ' ')
            print("Mowed: ", indego.state.mowed)
            

            
            
            
            f.write(str(logtime) + " - ")
            f.write(str(indego.state.mowed) + "\n")
            f.close()
            time.sleep(60)   # Delays for 5 seconds. You can also use a float value.

        # await indego.update_state(force=True)
        # print("state: ", indego.state)
        # print("Force xPos: ", indego.state.xPos)
        # print("Force yPos: ", indego.state.yPos)

        # Must call regular get_Sate before longpoll
        # await indego.update_state()
        # print("state: ", indego.state)
        # await indego.update_state(longpoll=True, longpoll_timeout=10)
        # print("state: ", indego.state)
        # print("state description: ", indego.state_description)
        # print("state description detail: ", indego.state_description_detail)

        
        # await indego.update_updates_available()
        # print("update available: ", indego.update_available)

        # await indego.update_user()
        # print("user: ", indego.user)



if __name__ == "__main__":
    with open("config.json", "r") as config_file:
        config = json.load(config_file)
    # config.pop("serial")
    # config["username"] = "sdjbfajhbsdf"
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main(config))



"""Test the states of pyIndego."""
import asyncio
import logging
from datetime import datetime
from socket import error as SocketError

import pytest
from aiohttp import (
    ClientOSError,
    ClientResponseError,
    ServerTimeoutError,
    TooManyRedirects,
    web,
)
from aiohttp.web_exceptions import HTTPGatewayTimeout
from mock import MagicMock, patch
from requests.exceptions import HTTPError, RequestException, Timeout
from requests.exceptions import TooManyRedirects as reqTooManyRedirects

from pyIndego import IndegoAsyncClient, IndegoClient
from pyIndego.const import CONTENT_TYPE, CONTENT_TYPE_JSON, Methods
from pyIndego.helpers import convert_bosch_datetime
from pyIndego.states import (
    Alert,
    Battery,
    Calendar,
    CalendarDay,
    CalendarSlot,
    Config,
    Garden,
    GenericData,
    Location,
    Network,
    OperatingData,
    PredictiveSchedule,
    Runtime,
    RuntimeDetail,
    Security,
    Setup,
    State,
    User,
)

_LOGGER = logging.getLogger(__name__)

alert = {
    "alm_sn": "test_sn",
    "alert_id": "5efda84ffbf591182723be89",
    "error_code": "104",
    "headline": "Mower requires attention.",
    "date": "2020-07-02T09:26:39.589Z",
    "message": "Stop button activated. The Stop button has been activated. Please follow the instructions on the mower display.",
    "read_status": "read",
    "flag": "warning",
    "push": True,
}
calendar = {
    "cal": 3,
    "days": [
        {
            "day": 0,
            "slots": [
                {"En": True, "StHr": 10, "StMin": 0, "EnHr": 13, "EnMin": 0},
                {"En": False},
            ],
        },
        {
            "day": 2,
            "slots": [
                {"En": True, "StHr": 11, "StMin": 0, "EnHr": 14, "EnMin": 0},
                {"En": False},
            ],
        },
        {
            "day": 4,
            "slots": [
                {"En": True, "StHr": 10, "StMin": 0, "EnHr": 13, "EnMin": 0},
                {"En": False},
            ],
        },
    ],
}
setup_json = {
    "hasOwner": True,
    "hasPin": True,
    "hasMap": True,
    "hasAutoCal": False,
    "hasIntegrityCheckPassed": True,
}
location = {"latitude": "1.1234", "longitude": "1.1234", "timezone": "Europe/Amsterdam"}
user = {
    "email": "test@test.com",
    "display_name": "test",
    "language": "en",
    "country": "NL",
    "optIn": False,
    "optInApp": False,
}
security = {"enabled": True, "autolock": False}
generic = {
    "alm_sn": "test_sn",
    "service_counter": 69272,
    "needs_service": False,
    "alm_mode": "smart",
    "bareToolnumber": "3600HB0102",
    "alm_firmware_version": "17329.01211",
}
lastcutting = {"last_mowed": "2020-06-29T12:24:03.664+02:00"}
nextcutting = {"mow_next": "2020-07-03T10:00:00+02:00"}
network = {"mcc": 204, "mnc": 16, "rssi": -83}
config = {
    "region": 0,
    "language": 14,
    "border_cut": 0,
    "is_pin_set": True,
    "wire_id": 4,
    "bump_sensitivity": 0,
    "alarm_mode": False,
}
operating = {
    "runtime": {
        "total": {"operate": 81106, "charge": 11834},
        "session": {"operate": 12, "charge": 12},
    },
    "battery": {
        "voltage": 8.6,
        "cycles": 1,
        "discharge": 0.0,
        "ambient_temp": 23,
        "battery_temp": 23,
        "percent": 86,
    },
    "garden": {
        "id": 34,
        "name": 1,
        "signal_id": 4,
        "size": 80,
        "inner_bounds": 238,
        "cuts": 61172,
        "runtime": 12,
        "charge": 11834,
        "bumps": 183,
        "stops": 10,
        "last_mow": 6,
        "map_cell_size": 120,
    },
    "hmiKeys": 213,
}
predictive_calendar = {
    "sel_cal": 1,
    "cals": [
        {
            "cal": 1,
            "days": [
                {
                    "day": 0,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 1,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 2,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 3,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 4,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 5,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 6,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
            ],
        }
    ],
}

predictive_schedule = {
    "schedule_days": [
        {
            "day": 1,
            "slots": [{"En": True, "StHr": 10, "StMin": 0, "EnHr": 13, "EnMin": 0}],
        },
        {
            "day": 3,
            "slots": [{"En": True, "StHr": 11, "StMin": 0, "EnHr": 14, "EnMin": 0}],
        },
        {
            "day": 5,
            "slots": [{"En": True, "StHr": 10, "StMin": 0, "EnHr": 13, "EnMin": 0}],
        },
    ],
    "exclusion_days": [
        {
            "day": 0,
            "slots": [
                {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0, "Attr": "C"},
                {
                    "En": True,
                    "StHr": 20,
                    "StMin": 0,
                    "EnHr": 23,
                    "EnMin": 59,
                    "Attr": "C",
                },
            ],
        },
        {
            "day": 1,
            "slots": [
                {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0, "Attr": "C"},
                {
                    "En": True,
                    "StHr": 20,
                    "StMin": 0,
                    "EnHr": 23,
                    "EnMin": 59,
                    "Attr": "C",
                },
            ],
        },
        {
            "day": 2,
            "slots": [
                {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0, "Attr": "C"},
                {
                    "En": True,
                    "StHr": 8,
                    "StMin": 0,
                    "EnHr": 20,
                    "EnMin": 0,
                    "Attr": "P",
                },
                {
                    "En": True,
                    "StHr": 20,
                    "StMin": 0,
                    "EnHr": 23,
                    "EnMin": 59,
                    "Attr": "C",
                },
            ],
        },
        {
            "day": 3,
            "slots": [
                {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0, "Attr": "C"},
                {
                    "En": True,
                    "StHr": 8,
                    "StMin": 0,
                    "EnHr": 11,
                    "EnMin": 0,
                    "Attr": "p",
                },
                {
                    "En": True,
                    "StHr": 20,
                    "StMin": 0,
                    "EnHr": 23,
                    "EnMin": 59,
                    "Attr": "C",
                },
            ],
        },
        {
            "day": 4,
            "slots": [
                {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0, "Attr": "C"},
                {
                    "En": True,
                    "StHr": 8,
                    "StMin": 0,
                    "EnHr": 11,
                    "EnMin": 0,
                    "Attr": "Pp",
                },
                {
                    "En": True,
                    "StHr": 20,
                    "StMin": 0,
                    "EnHr": 23,
                    "EnMin": 59,
                    "Attr": "C",
                },
            ],
        },
        {
            "day": 5,
            "slots": [
                {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0, "Attr": "C"},
                {
                    "En": True,
                    "StHr": 20,
                    "StMin": 0,
                    "EnHr": 23,
                    "EnMin": 59,
                    "Attr": "C",
                },
            ],
        },
        {
            "day": 6,
            "slots": [
                {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0, "Attr": "C"},
                {"En": True, "StHr": 8, "StMin": 0, "EnHr": 9, "EnMin": 0, "Attr": "D"},
                {
                    "En": True,
                    "StHr": 20,
                    "StMin": 0,
                    "EnHr": 23,
                    "EnMin": 59,
                    "Attr": "C",
                },
            ],
        },
    ],
}

state = {
    "state": 64513,
    "map_update_available": True,
    "mowed": 97,
    "mowmode": 2,
    "xPos": 5,
    "yPos": 50,
    "runtime": {
        "total": {"operate": 81329, "charge": 11912},
        "session": {"operate": 10, "charge": 0},
    },
    "mapsvgcache_ts": 1593609416617,
    "svg_xPos": 720,
    "svg_yPos": 424,
    "config_change": False,
    "mow_trig": True,
}

test_config = {"username": "testname", "password": "testpassword", "api_url": "", "serial": "123456789"}


class AsyncMock(MagicMock):
    async def __call__(self, *args, **kwargs):
        return super(AsyncMock, self).__call__(*args, **kwargs)


class SyncMock(MagicMock):
    def __call__(self, *args, **kwargs):
        return super(SyncMock, self).__call__(*args, **kwargs)


class MockResponseAsync:
    """Class for mock responses in async."""

    def __init__(self, json, status):
        """Init the async mock response."""
        self._json = json
        self.status = status

    async def json(self):
        """Return json."""
        return self._json

    @property
    def content_type(self):
        """Return content type."""
        return CONTENT_TYPE_JSON

    async def __aexit__(self, exc_type, exc, tb):
        """Do async exit."""
        pass

    async def __aenter__(self):
        """Do async enter."""
        return self


class MockResponseSync:
    """Class for mock responses in sync."""

    def __init__(self, json, status):
        """Init the sync mock response."""
        self._json = json
        self.status_code = status

    def json(self):
        """Return json."""
        return self._json

    @property
    def headers(self):
        """Return header."""
        return {CONTENT_TYPE: f"{CONTENT_TYPE_JSON};"}


class TestIndego(object):
    """States class."""

    def test_repr(self):
        """Test the representation string."""
        indego = IndegoClient(**test_config)
        str(indego)

    @pytest.mark.parametrize(
        "state, json, checks",
        [
            (Alert, alert, ["alm_sn"]),
            (OperatingData, operating, ["hmiKeys", ("garden.id", "['garden']['id']")]),
            (
                Calendar,
                calendar,
                [
                    "cal",
                    ("days[0].day", "['days'][0]['day']"),
                    ("days[0].slots[0].En", "['days'][0]['slots'][0]['En']"),
                ],
            ),
            (State, state, ["state"]),
        ],
    )
    def test_states(self, state, json, checks):
        """Test States classes."""
        state_instance = state(**json)  # noqa: F841, pylint: disable=unused-variable
        for check in checks:
            if isinstance(check, str):
                value_state = eval(f"state_instance.{check}")
                value_json = eval(f"json['{check}']")
            else:
                value_state = eval(f"state_instance.{check[0]}")
                value_json = eval(f"json{check[1]}")
            assert value_state == value_json

    @pytest.mark.parametrize(
        "date_str, date_dt",
        [
            (
                "2020-07-01T13:22:43.15+02:00",
                datetime.fromisoformat("2020-07-01 13:22:43.150000+02:00"),
            ),
            (
                "2020-07-03T10:00:00+02:00",
                datetime.fromisoformat("2020-07-03 10:00:00+02:00"),
            ),
            (
                datetime.fromisoformat("2020-07-03 10:00:00+02:00"),
                datetime.fromisoformat("2020-07-03 10:00:00+02:00"),
            ),
            (None, None),
        ],
    )
    def test_date_parsing(self, date_str, date_dt):
        """Test the convert_bosch_datetime function."""
        test_dt = convert_bosch_datetime(date_str)
        assert test_dt == date_dt

    @pytest.mark.parametrize(
        "sync, func, attr, ret_value, assert_value",
        [
            (
                True,
                IndegoClient.update_calendar,
                "calendar",
                {"sel_cal": 3, "cals": [calendar]},
                Calendar(**calendar),
            ),
            (True, IndegoClient.update_alerts, "alerts", [alert], [Alert(**alert)]),
            (True, IndegoClient.update_config, "config", config, Config(**config)),
            (
                True,
                IndegoClient.update_generic_data,
                "generic_data",
                generic,
                GenericData(**generic),
            ),
            (
                True,
                IndegoClient.update_last_completed_mow,
                "last_completed_mow",
                {"last_mowed": "2020-07-01T13:22:43.15+02:00"},
                datetime.fromisoformat("2020-07-01 13:22:43.150000+02:00"),
            ),
            (
                True,
                IndegoClient.update_location,
                "location",
                location,
                Location(**location),
            ),
            (True, IndegoClient.update_network, "network", network, Network(**network)),
            (
                True,
                IndegoClient.update_next_mow,
                "next_mow",
                {"mow_next": "2020-07-03T10:00:00+02:00"},
                datetime.fromisoformat("2020-07-03 10:00:00+02:00"),
            ),
            (
                True,
                IndegoClient.update_operating_data,
                "operating_data",
                operating,
                OperatingData(**operating),
            ),
            (
                True,
                IndegoClient.update_predictive_calendar,
                "predictive_calendar",
                predictive_calendar,
                Calendar(**predictive_calendar["cals"][0]),
            ),
            (
                True,
                IndegoClient.update_predictive_schedule,
                "predictive_schedule",
                predictive_schedule,
                PredictiveSchedule(**predictive_schedule),
            ),
            (
                True,
                IndegoClient.update_security,
                "security",
                security,
                Security(**security),
            ),
            (True, IndegoClient.update_setup, "setup", setup_json, Setup(**setup_json)),
            (True, IndegoClient.update_state, "state", state, State(**state)),
            (
                True,
                IndegoClient.update_updates_available,
                "update_available",
                {"available": False},
                False,
            ),
            (True, IndegoClient.update_user, "user", user, User(**user)),
            (True, IndegoClient.update_all, "user", None, None),
            (
                False,
                IndegoAsyncClient.update_calendar,
                "calendar",
                {"sel_cal": 3, "cals": [calendar]},
                Calendar(**calendar),
            ),
            (
                False,
                IndegoAsyncClient.update_alerts,
                "alerts",
                [alert],
                [Alert(**alert)],
            ),
            (
                False,
                IndegoAsyncClient.update_config,
                "config",
                config,
                Config(**config),
            ),
            (
                False,
                IndegoAsyncClient.update_generic_data,
                "generic_data",
                generic,
                GenericData(**generic),
            ),
            (
                False,
                IndegoAsyncClient.update_last_completed_mow,
                "last_completed_mow",
                {"last_mowed": "2020-07-01T13:22:43.15+02:00"},
                datetime.fromisoformat("2020-07-01 13:22:43.150000+02:00"),
            ),
            (
                False,
                IndegoAsyncClient.update_location,
                "location",
                location,
                Location(**location),
            ),
            (
                False,
                IndegoAsyncClient.update_network,
                "network",
                network,
                Network(**network),
            ),
            (
                False,
                IndegoAsyncClient.update_next_mow,
                "next_mow",
                {"mow_next": "2020-07-03T10:00:00+02:00"},
                datetime.fromisoformat("2020-07-03 10:00:00+02:00"),
            ),
            (
                False,
                IndegoAsyncClient.update_operating_data,
                "operating_data",
                operating,
                OperatingData(**operating),
            ),
            (
                False,
                IndegoAsyncClient.update_predictive_calendar,
                "predictive_calendar",
                predictive_calendar,
                Calendar(**predictive_calendar["cals"][0]),
            ),
            (
                False,
                IndegoAsyncClient.update_predictive_schedule,
                "predictive_schedule",
                predictive_schedule,
                PredictiveSchedule(**predictive_schedule),
            ),
            (
                False,
                IndegoAsyncClient.update_security,
                "security",
                security,
                Security(**security),
            ),
            (
                False,
                IndegoAsyncClient.update_setup,
                "setup",
                setup_json,
                Setup(**setup_json),
            ),
            (False, IndegoAsyncClient.update_state, "state", state, State(**state)),
            (
                False,
                IndegoAsyncClient.update_updates_available,
                "update_available",
                {"available": False},
                False,
            ),
            (False, IndegoAsyncClient.update_user, "user", user, User(**user)),
            (False, IndegoAsyncClient.update_all, "user", None, None),
        ],
    )
    async def test_client_update_functions(
        self, sync, func, attr, ret_value, assert_value
    ):
        """Test the base client functions with 200."""
        if sync:
            resp = MockResponseSync(ret_value, 200)
            with patch("requests.request", return_value=resp):
                indego = IndegoClient(**test_config)
                indego._online = True
                indego._userid = "test_user_id"
                func(indego)
                assert getattr(indego, attr) == assert_value
                if attr == "state":
                    assert indego.state_description == "Docked"
                    assert indego.state_description_detail == "Sleeping"
        else:
            resp = MockResponseAsync(ret_value, 200)
            with patch("aiohttp.ClientSession.request", return_value=resp), patch(
                "pyIndego.IndegoAsyncClient.start", return_value=True
            ):
                async with IndegoAsyncClient(**test_config) as indego:
                    indego._contextid = "askdjfbaks"
                    indego._online = True
                    indego._userid = "test_user_id"
                    await func(indego)
                    assert getattr(indego, attr) == assert_value
                    if attr == "state":
                        assert indego.state_description == "Docked"
                        assert indego.state_description_detail == "Sleeping"

    @pytest.mark.parametrize(
        "sync, func, param, attr, ret_value, assert_value",
        [
            (
                True,
                IndegoClient.update_state,
                {"force": True},
                "state",
                state,
                State(**state),
            ),
            (
                True,
                IndegoClient.update_state,
                {"longpoll": True},
                "state",
                state,
                State(**state),
            ),
            (
                True,
                IndegoClient.update_state,
                {"force": True, "longpoll": True},
                "state",
                state,
                State(**state),
            ),
            (
                False,
                IndegoAsyncClient.update_state,
                {"force": True},
                "state",
                state,
                State(**state),
            ),
            (
                False,
                IndegoAsyncClient.update_state,
                {"longpoll": True},
                "state",
                state,
                State(**state),
            ),
            (
                False,
                IndegoAsyncClient.update_state,
                {"force": True, "longpoll": True},
                "state",
                state,
                State(**state),
            ),
        ],
    )
    async def test_client_update_state_params(
        self, sync, func, param, attr, ret_value, assert_value
    ):
        """Test the base client functions with 200."""
        if sync:
            resp = MockResponseSync(ret_value, 200)
            with patch("requests.request", return_value=resp):
                indego = IndegoClient(**test_config)
                indego._online = True
                indego._userid = "test_user_id"
                func(indego, **param)
                assert getattr(indego, attr) == assert_value
        else:
            resp = MockResponseAsync(ret_value, 200)
            with patch("aiohttp.ClientSession.request", return_value=resp), patch(
                "pyIndego.IndegoAsyncClient.start", return_value=True
            ):
                async with IndegoAsyncClient(**test_config) as indego:
                    indego._contextid = "askdjfbaks"
                    indego._online = True
                    indego._userid = "test_user_id"
                    await func(indego, **param)
                    assert getattr(indego, attr) == assert_value

    @pytest.mark.parametrize(
        "sync, func, attr, ret_value, assert_value",
        [
            (True, IndegoClient.update_user, "user", user, User(**user)),
            (False, IndegoAsyncClient.update_user, "user", user, User(**user)),
        ],
    )
    async def test_client_replace(self, sync, func, attr, ret_value, assert_value):
        """Test the base client functions with 200."""
        if sync:
            resp = MockResponseSync(ret_value, 200)
            with patch("requests.request", return_value=resp):
                indego = IndegoClient(**test_config)
                indego._online = True
                indego._userid = "test_user_id"
                func(indego)
                assert getattr(indego, attr) == assert_value
                func(indego)
                assert getattr(indego, attr) == assert_value
        else:
            resp = MockResponseAsync(ret_value, 200)
            with patch("aiohttp.ClientSession.request", return_value=resp), patch(
                "pyIndego.IndegoAsyncClient.start", return_value=True
            ):
                async with IndegoAsyncClient(**test_config) as indego:
                    indego._contextid = "askdjfbaks"
                    indego._online = True
                    indego._userid = "test_user_id"
                    await func(indego)
                    assert getattr(indego, attr) == assert_value
                    await func(indego)
                    assert getattr(indego, attr) == assert_value

    @pytest.mark.parametrize(
        "sync, response, func, attr, ret_value",
        [
            (True, 204, IndegoClient.update_user, "user", user),
            (True, 400, IndegoClient.update_user, "user", user),
            (True, 401, IndegoClient.update_user, "user", user),
            (True, 403, IndegoClient.update_user, "user", user),
            (True, 405, IndegoClient.update_user, "user", user),
            (True, 501, IndegoClient.update_user, "user", user),
            (True, 504, IndegoClient.update_user, "user", user),
            (False, 204, IndegoAsyncClient.update_user, "user", user),
            (False, 400, IndegoAsyncClient.update_user, "user", user),
            (False, 401, IndegoAsyncClient.update_user, "user", user),
            (False, 403, IndegoAsyncClient.update_user, "user", user),
            (False, 405, IndegoAsyncClient.update_user, "user", user),
            (False, 501, IndegoAsyncClient.update_user, "user", user),
            (False, 504, IndegoAsyncClient.update_user, "user", user),
        ],
    )
    async def test_client_responses(self, sync, response, func, attr, ret_value):
        """Test the request functions with different responses."""
        if sync:
            resp = MockResponseSync(ret_value, response)
            with patch("requests.request", return_value=resp):
                indego = IndegoClient(**test_config)
                indego._online = True
                indego._userid = "test_user_id"
                func(indego)
                assert getattr(indego, attr) == None
        else:
            resp = MockResponseAsync(ret_value, response)
            with patch("aiohttp.ClientSession.request", return_value=resp):
                async with IndegoAsyncClient(**test_config) as indego:
                    indego._online = True
                    indego._userid = "test_user_id"
                    await func(indego)
                    assert getattr(indego, attr) == None

    @pytest.mark.parametrize(
        "error",
        [
            (asyncio.CancelledError),
            (Exception),
            (asyncio.TimeoutError),
            (ServerTimeoutError),
            (HTTPGatewayTimeout),
            (ClientOSError),
            (TooManyRedirects),
            (ClientResponseError),
            (SocketError),
        ],
    )
    async def test_a_client_response_errors(self, error):
        """Test the request functions with different responses."""
        with patch("aiohttp.ClientSession.request", side_effect=error), patch(
            "asyncio.sleep", new_callable=AsyncMock
        ):
            async with IndegoAsyncClient(**test_config) as indego:
                indego._online = True
                indego._userid = "test_user_id"
                resp = await indego._request(
                    method=Methods.GET, path="alerts", timeout=1
                )
                assert resp is None

    @pytest.mark.parametrize(
        "error", [(Exception), (Timeout), (reqTooManyRedirects), (RequestException)]
    )
    def test_client_response_errors(self, error):
        """Test the request functions with different responses."""
        with patch("requests.request", side_effect=error), patch(
            "time.sleep", new_callable=SyncMock
        ):
            indego = IndegoClient(**test_config)
            indego._online = True
            indego._userid = "test_user_id"
            resp = indego._request(method=Methods.GET, path="alerts", timeout=1)
            assert resp is None

    @pytest.mark.parametrize(  # noqa: ignore:C901
        "alerts, loaded, index, error",
        [
            ([Alert(**alert)], True, 0, None),
            ([Alert(**alert)], True, 1, IndexError),
            (None, True, 0, ValueError),
            (None, False, 0, ValueError),
        ],
    )
    async def test_alert_functions(self, alerts, loaded, index, error):
        """Test the function for handling alerts."""
        resp = MockResponseSync(True, 200)
        with patch("requests.request", return_value=resp):
            indego = IndegoClient(**test_config)
            indego._online = True
            indego._userid = "test_user_id"
            indego.alerts = alerts
            indego._alerts_loaded = loaded
            try:
                res = indego.delete_alert(index)
                if error and not loaded:
                    assert False
                elif not alerts and loaded:
                    assert res is None
                else:
                    assert res
            except error:
                assert True
            try:
                res = indego.put_alert_read(index)
                if error and not loaded:
                    assert False
                elif not alerts and loaded:
                    assert res is None
                else:
                    assert res
            except error:
                assert True
            try:
                res = indego.delete_all_alerts()
                if alerts is None and loaded:
                    assert res is None
                else:
                    assert res
            except error:
                assert True
            try:
                res = indego.put_all_alerts_read()
                if alerts is None and loaded:
                    assert res is None
                else:
                    assert res
            except error:
                assert True
        resp = MockResponseAsync(True, 200)
        with patch("aiohttp.ClientSession.request", return_value=resp), patch(
            "pyIndego.IndegoAsyncClient.start", return_value=True
        ):
            async with IndegoAsyncClient(**test_config) as indego:
                indego._online = True
                indego._userid = "test_user_id"
                indego.alerts = alerts
                indego._alerts_loaded = loaded
                try:
                    res = await indego.delete_alert(index)
                    if error and not loaded:
                        assert False
                    elif not alerts and loaded:
                        assert res is None
                    else:
                        assert res
                except error:
                    assert True
                try:
                    res = await indego.put_alert_read(index)
                    if error and not loaded:
                        assert False
                    elif not alerts and loaded:
                        assert res is None
                    else:
                        assert res
                except error:
                    assert True
                try:
                    res = await indego.delete_all_alerts()
                    if alerts is None and loaded:
                        assert res is None
                    else:
                        assert res
                except error:
                    assert True
                try:
                    res = await indego.put_all_alerts_read()
                    if alerts is None and loaded:
                        assert res is None
                    else:
                        assert res
                except error:
                    assert True

    @pytest.mark.parametrize(  # noqa: ignore:C901
        "command, param, error",
        [
            ("command", "mow", None),
            ("command", "pause", None),
            ("command", "returnToDock", None),
            ("command", "mows", ValueError),
            ("mow_mode", "true", None),
            ("mow_mode", "false", None),
            ("mow_mode", "True", None),
            ("mow_mode", "False", None),
            ("mow_mode", True, None),
            ("mow_mode", False, None),
            ("mow_mode", "mows", ValueError),
            ("pred_cal", None, None),
            ("pred_cal", {"cals": 1}, ValueError),
        ],
    )
    async def test_commands(self, command, param, error):
        """Test the function for handling alerts."""
        resp = MockResponseSync(True, 200)
        with patch("requests.request", return_value=resp):
            indego = IndegoClient(**test_config)
            indego._online = True
            indego._userid = "test_user_id"
            if command == "command":
                try:
                    indego.put_command(param)
                    if error:
                        assert False
                    assert True
                except error:
                    assert True
            elif command == "mow_mode":
                try:
                    indego.put_mow_mode(param)
                    if error:
                        assert False
                    assert True
                except error:
                    assert True
            elif command == "pred_cal":
                try:
                    if param:
                        indego.put_predictive_cal(param)
                    else:
                        indego.put_predictive_cal()
                    if error:
                        assert False
                    assert True
                except error:
                    assert True

        resp = MockResponseAsync(True, 200)
        with patch("aiohttp.ClientSession.request", return_value=resp), patch(
            "pyIndego.IndegoAsyncClient.start", return_value=True
        ):
            async with IndegoAsyncClient(**test_config) as indego:
                indego._online = True
                indego._userid = "test_user_id"
                if command == "command":
                    try:
                        await indego.put_command(param)
                        if error:
                            assert False
                        assert True
                    except error:
                        assert True
                elif command == "mow_mode":
                    try:
                        await indego.put_mow_mode(param)
                        if error:
                            assert False
                        assert True
                    except error:
                        assert True
                elif command == "pred_cal":
                    try:
                        if param:
                            await indego.put_predictive_cal(param)
                        else:
                            await indego.put_predictive_cal()
                        if error:
                            assert False
                        assert True
                    except error:
                        assert True

    @pytest.mark.parametrize(
        "config",
        [
            ({"username": "testname", "password": "testpassword", "api_url": ""}),
            (
                {
                    "username": "testname",
                    "password": "testpassword",
                    "serial": "123456789",
                    "api_url": "",
                }
            ),
        ],
    )
    async def test_login(self, config):
        """Test the function for handling alerts."""
        resp_json = {"contextId": "98765", "userId": "12345"}
        resp_get = [{"alm_sn": "123456789"}]
        resp_login_s = MockResponseSync(resp_json, 200)
        with patch("requests.request", return_value=resp_login_s), patch(
            "pyIndego.IndegoClient.get", return_value=resp_get
        ):
            indego = IndegoClient(**config)
            indego.login()
            assert indego._userid == "12345"
            assert indego.serial == "123456789"

        resp_login_a = MockResponseAsync(resp_json, 200)
        with patch("aiohttp.ClientSession.request", return_value=resp_login_a), patch(
            "pyIndego.IndegoAsyncClient.start", return_value=True
        ), patch("pyIndego.IndegoAsyncClient.get", return_value=resp_get):
            async with IndegoAsyncClient(**config) as indego:
                await indego.login()
                assert indego._userid == "12345"
                assert indego.serial == "123456789"

    @pytest.mark.parametrize(
        "config, param, error",
        [(None, None, ValueError), (None, "test.svg", None), ("test.svg", None, None)],
    )
    async def test_download(self, config, param, error):
        """Test the function for download map."""
        conf = test_config.copy()
        if config:
            conf.update({"map_filename": config})
        with patch("pyIndego.IndegoClient.get", return_value=None):
            indego = IndegoClient(**conf)
            try:
                indego.download_map(param)
                assert indego.map_filename == "test.svg"
                if error:
                    assert False
            except error:
                assert True

        with patch("pyIndego.IndegoAsyncClient.start", return_value=True), patch(
            "pyIndego.IndegoAsyncClient.get", return_value=None
        ):
            async with IndegoAsyncClient(**conf) as indego:
                try:
                    await indego.download_map(param)
                    assert indego.map_filename == "test.svg"
                    if error:
                        assert False
                except error:
                    assert True

    def test_update_battery(self):
        """Test the battery update function."""
        indego = IndegoClient(**test_config)
        indego._update_generic_data(generic)
        indego._update_operating_data(operating)
